/*-------------------------------------------------------------*/
/*   x11intf: definition of actions                            */
/*-------------------------------------------------------------*/
/*                                                             */
/* action                                                      */
/* -1 : Store config-file name in static array                 */
/*  0 : close the window                                       */
/*  1 : open the window                                        */
/*  2 : draw from x1, y1 to x2, y2                             */
/*  3 : reset title and menu window                            */
/*  5 : eventloop (mouse)                                      */
/*  6 : clear screen                                           */
/*  7 : xflush                                                 */
/*  8 : Set Switches to default value                          */
/*  9 : draw string (z chars) in position x,y                  */
/* 10 : draw circle center in  x,y ,radius = z                 */
/* 11 : fill circle center in  x,y ,radius = z                 */
/* 12 : set foreground color to col                            */
/* 13 : draw z-points polygon using color col                  */
/* 14 : fill z-points polygon using color col                  */
/* 15 : Print a message at the bottom of the window            */
/* 16 : Store atomic species in static array                   */
/*-------------------------------------------------------------*/

/*-------------------------------------------------------------*/
/*          Include files                                      */
/*-------------------------------------------------------------*/
/* CHANGE  following include to be used ONLY on VMS computers  */
/* OTHERWISE use standard X11 include                          */

/*
#include "sys$common:[decw$include]Xlib.h"
#include "sys$common:[decw$include]Xutil.h"
#include "sys$common:[decw$include]Xos.h"
#include "sys$common:[decw$include]Intrinsic.h"
#include "sys$common:[decw$include]Xatom.h"
*/


#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>

#include <stdio.h>
#include <stdlib.h>

/*-------------------------------------------------------------*/
/*          Constants  definition                              */
/*-------------------------------------------------------------*/
#define MAX_COLORS  16
#define BOXES  41
#define BOXT   78
#define BOXS   11

/*-------------------------------------------------------------*/
/*          Functions declaration                              */
/*-------------------------------------------------------------*/
void SelectAtms ();
void DrawMenu ();
void DrawSpecies ();
void CreateHelp ();
void Tasto ();
void C_PrintMess ();
void PrintImageMess ();
void Bottom();
void ClearBottom();
void ClearMenu();
void TopLeft ();
void Sorcio ();
void Seleziona ();
void HardCopy ();
void ViewPlane ();
void Geometry ();
void RelAt ();
void RedrawTasto ();
int KeBox ();
Boolean IsAFile ();
void LoadFonts ();
static Boolean First = True,histo = False;
static XRectangle drawing,all;

/*-------------------------------------------------------------*/
/*      Static variables (to save value)                       */
/*-------------------------------------------------------------*/
static char Titolo[81],filcon[81];
static char Species[81];
static char Species_copy[81];
static int NumSpec;
static int IfEdges, IfScale, IfCont;
static Display *display;
static GC     gc;                  
static Boolean Monocro = FALSE;
static Boolean hc = FALSE;
static Boolean FMenu = FALSE, FHelp = FALSE, FMesg = FALSE;
static int nbox0,nboxn,colors[MAX_COLORS],white,black,old_color,new_color;
static int curstyl = 2, helpcol = 14;
static int BW = 20 , BH = 50 ;
static int menucont = 4,menulab = 4,botrig = 1,toplef = 11,titcol = 4;
static int ipres,jpres,kprog,khead,deltax,deltay,kscrx,kscry;
static int xbox[BOXT] = { 
    0,  /* Plot style            0 */
    0,  /* Rotate                1 */
   20,  /*   X                   2 */
   20,  /*   Y                   3 */
   20,  /*   Z                   4 */
    0,  /* Translate             5 */
   20,  /*   X                   6 */
   20,  /*   Y                   7 */
   20,  /*   Z                   8 */
    0,  /* Quit                  9 */
    0,  /*   1                   10 */
   27,  /*   2                   11 */
   54,  /*   3                   12 */
    0,  /*   4                   13 */
   27,  /*   5                   14 */
   54,  /*   6                   15 */
    0,  /*   -                   16 */
   53,  /*   +                   17 */
    0,  /*   -                   18 */
   53,  /*   +                   19 */
    0,  /*   -                   20 */
   53,  /*   +                   21 */
    0,  /*   -                   22 */
   53,  /*   +                   23 */
    0,  /*   -                   24 */
   53,  /*   +                   25 */
    0,  /*   -                   26 */
   53,  /*   +                   27 */
    0,  /* Edges on              28 */
    0,  /* Symm  on              29 */
    0,  /* Scale off             30 */
    0,  /* Delete                31 */
    0,  /* Select                32 */
    0,  /* View plane            33 */
    0,  /* Relabel               34 */
    0,  /* Regroup               35 */
    0,  /* Refine                36 */
    0,  /* Complete              37 */
    0,  /* Help                  38 */
   579, /* Hardcopy              39 */
    0,  /* Geometry              40 */
   170, /* To delete             41 */
   270, /* To hide               42 */
   370, /* To display            43 */
   480, /* Show all              44 */
   579, /* Cancel                45 */
   460, /* Close Pol.            46 */
   579, /* Cancel                47 */
    90, /* Least sq.             48 */
   210, /* N. to L&M             49 */
   340, /* Most sq.              50 */
   470, /* More                  51 */
   579, /* Cancel                52 */
    90, /* Norm. to A            53 */
   210, /* Norm. to B            54 */
   340, /* Norm. to C            55 */
   470, /* More                  56 */
   579, /* Cancel                57 */
   390, /*                       58 */
   570, /*                       59 */
   579, /*                       60 */
    85, /* Distances             61 */
   205, /* Angles                62 */
   325, /* Torsion Angles        63 */
   465, /* More                  64 */
   579, /* Cancel                65 */
    85, /* Add Bonds             66 */
   205, /* Delete Bonds          67 */
   325, /* Info                  68 */
   465, /* More                  69 */
   579, /* Cancel                70 */
   366, /* Up                    71 */
   419, /* Down                  72 */
   479, /* Cancel                73 */
   549, /* Bar Chart             74 */
   466, /* Up   - Disp Dist      75 */
   519, /* Down - Disp Dist      76 */
   579  /* Cancel - Disp Dist    77 */
};
static int ybox[BOXT] = { 
   20,  /* Plot style            0 */
    80,  /* Rotate                1 */
   100,  /*   X                   2 */
   120,  /*   Y                   3 */
   140,  /*   Z                   4 */
   160,  /* Translate             5 */
   180,  /*   X                   6 */
   200,  /*   Y                   7 */
   220,  /*   Z                   8 */
    0,  /* Quit                  9 */
   40,  /*   1                   10 */
   40,  /*   2                   11 */
   40,  /*   3                   12 */
   60,  /*   4                   13 */
   60,  /*   5                   14 */
   60,  /*   6                   15 */
   100,  /*   -                   16 */
   100,  /*   +                   17 */
   120,  /*   -                   18 */
   120,  /*   +                   19 */
   140,  /*   -                   20 */
   140,  /*   +                   21 */
   180,  /*   -                   22 */
   180,  /*   +                   23 */
   200,  /*   -                   24 */
   200,  /*   +                   25 */
   220,  /*   -                   26 */
   220,  /*   +                   27 */
   240,  /* Edges on              28 */
   260,  /* Symm  on              29 */
   280,  /* Scale off             30 */
   300,  /* Delete                31 */
   320,  /* Select                32 */
   340,  /* View plane            33 */
   360,  /* Relabel               34 */
   400,  /* Regroup               35 */
   420,  /* Refine                36 */
   440,  /* Complete              37 */
   460,  /* Help                  38 */
    0,  /* HardCopy               39 */
   380, /* Geometry               40 */
   459, /* To delete              41 */
   459, /* To hide                42 */
   459, /* To display             43 */
   459, /* Show all               44 */
   459, /* Cancel                 45 */
   459, /* Close Pol.             46 */
   459, /* Cancel                 47 */
   459, /* Least sq.              48 */
   459, /* N. to L&M              49 */
   459, /* Most sq.               50 */
   459, /* More                   51 */
   459, /* Cancel                 52 */
   459, /* Norm. to A             53 */
   459, /* Norm. to B             54 */
   459, /* Norm. to C             55 */
   459, /* More                   56 */
   459, /* Cancel                 57 */
   459, /*                       58 */
   459, /*                       59 */
   459, /*                       60 */
   459, /* Distances             61 */
   459, /* Angles                62 */
   459, /* Torsion Angles        63 */
   459, /* More                  64 */
   459, /* Cancel                65 */
   459, /* Add Bonds             66 */
   459, /* Delete Bonds          67 */
   459, /* Info                  68 */
   459, /* More                  69 */
   459, /* Cancel                70 */
   459, /* Up                    71 */
   459, /* Down                  72 */
   459, /* Cancel                73 */
   459, /* Bar Chart             74 */
   459, /* Up - Disp Dist        75 */
   459, /* Down - Disp Dist      76 */
   459  /* Cancel - Disp Dist    77 */
};
static int lbox[BOXT] = { 
   80,  /* Plot style            0 */
   80,  /* Rotate                1 */
   40,  /*   X                   2 */
   40,  /*   Y                   3 */
   40,  /*   Z                   4 */
   80,  /* Translate             5 */
   40,  /*   X                   6 */
   40,  /*   Y                   7 */
   40,  /*   Z                   8 */
   80,  /* Quit                  9 */
   27,  /*   1                   10 */
   27,  /*   2                   11 */
   26,  /*   3                   12 */
   27,  /*   4                   13 */
   27,  /*   5                   14 */
   26,  /*   6                   15 */
   27,  /*   +                   16 */
   27,  /*   -                   17 */
   27,  /*   +                   18 */
   27,  /*   -                   19 */
   27,  /*   +                   20 */
   27,  /*   -                   21 */
   27,  /*   +                   22 */
   27,  /*   -                   23 */
   27,  /*   +                   24 */
   27,  /*   -                   25 */
   27,  /*   +                   26 */
   27,  /*   -                   27 */
   80,  /* Edges on              28 */
   80,  /* Symm  on              29 */
   80,  /* Scale off             30 */
   80,  /* Delete                31 */
   80,  /* Select                32 */
   80,  /* View plane            33 */
   80,  /* Relabel               34 */
   80,  /* Regroup               35 */
   80,  /* Refine                36 */
   80,  /* Complete              37 */
   80,  /* Help                  38 */
   80,  /* HardCopy              39 */
   80,  /* Geometry              40 */
   80,  /* To delete             41 */
   80,  /* To hide               42 */
   90,  /* To display            43 */
   80,  /* Show all              44 */
   60,  /* Cancel                45 */
   100, /* Close Pol.            46 */
   60,  /* Cancel                47 */
   80,  /* Least sq.             48 */
   80,  /* N. to L&M             49 */
   80,  /* Most sq.              50 */
   80,  /* More                  51 */
   60,  /* Cancel                52 */
   80,  /* Norm. to A            53 */
   80,  /* Norm. to B            54 */
   80,  /* Norm. to C            55 */
   80,  /* More                  56 */
   60,  /* Cancel                57 */
   80, /*                       58 */
   80, /*                       59 */
   80, /*                       60 */
  100, /* Distances             61 */
  100, /* Angles                62 */
  100, /* Torsion Angles        63 */
   80, /* More                  64 */
   60, /* Cancel                65 */
  100, /* Add Bonds             66 */
  100, /* Delete Bonds          67 */
  100, /* Info                  68 */
   80, /* More                  69 */
   60, /* Cancel                70 */
   45, /* Up                    71 */
   50, /* Down                  72 */
   60, /* Cancel                73 */
   90, /* Bar Chart             74 */
   45, /* Up - Disp Dist        75 */
   50, /* Down - Disp Dist      76 */
   60  /* Cancel - Disp Dist    77 */
};
static Window mwindow;
static int width, height,wi,he;             
static XFontStruct *fontStruct, *fntmenu, *fnttab;
static char *strbox[BOXT] = { 
   " Plot Style",    /* 0 */
   "    Rotate  ",   /* 1 */
   "  X",            /* 2 */
   "  Y",            /* 3 */
   "  Z",            /* 4 */
   " Translate ",    /* 5 */
   "  X",            /* 6 */
   "  Y",            /* 7 */
   "  Z",            /* 8 */
   "      Quit  ",   /* 9 */
   "1",              /* 10 */
   "2",              /* 11 */
   "3",              /* 12 */
   "4",              /* 13 */
   "5",              /* 14 */
   "6",              /* 15 */
   "-",              /* 16 */
   "+",              /* 17 */
   "-",              /* 18 */
   "+",              /* 19 */
   "-",              /* 20 */
   "+",              /* 21 */
   "-",              /* 22 */
   "+",              /* 23 */
   "-",              /* 24 */
   "+",              /* 25 */
   "-",              /* 26 */
   "+",              /* 27 */
   " Edges on ",     /* 28 */
   " Symm  on ",     /* 29 */
   " Scale off",     /* 30 */
   "   Delete ",     /* 31 */
   "   Select ",     /* 32 */
   " View pl. ",     /* 33 */
   "   Relabel",     /* 34 */
   "   Regroup",     /* 35 */
   "   Refine ",     /* 36 */
   "  Complete",     /* 37 */
   "      Help  ",   /* 38 */
   "  HCopy     ",   /* 39 */
   "  Geometry  ",   /* 40 */
   " To delete  ",   /* 41 */
   " To hide    ",   /* 42 */
   " To display ",   /* 43 */
   " Show all   ",   /* 44 */
   " Cancel     ",   /* 45 */
   "Close Polygon",  /* 46 */
   " Cancel   ",     /* 47 */
   " Least Sq.",     /* 48 */
   " N. to L&M",     /* 49 */
   " Most Sq. ",     /* 50 */
   " More ... ",     /* 51 */
   " Cancel   ",     /* 52 */
   " Norm.to A",     /* 53 */
   " Norm.to B",     /* 54 */
   " Norm.to C",     /* 55 */
   " More ... ",     /* 56 */
   " Cancel   "      ,/* 57 */
   "               " ,/* 58 */
   "               " ,/* 59 */
   "               " ,/* 60 */
   "   Distances   " ,/* 61 */
   "     Angles    " ,/* 62 */
   "Torsion Angles " ,/* 63 */
   " More ...      " ,/* 64 */
   " Cancel        " ,/* 65 */
   "  Add Bonds    " ,/* 66 */
   " Delete Bonds  " ,/* 67 */
   "     Info      " ,/* 68 */
   " More ...      " ,/* 69 */
   " Cancel        " ,/* 70 */
   " Up            " ,/* 71 */
   " Down          " ,/* 72 */
   " Cancel        " ,/* 73 */
   " Bar Chart     " ,/* 74 */
   " Up            " ,/* 75 */
   " Down          " ,/* 76 */
   " Cancel        "  /* 77 */
};
static char *strapp[BOXS] =     { 
   " Edges off",
   " Edges on ",
   " Symm  off",
   " Symm  on ",
   " Scale off",
   " Scale on ",
   " Cancel   ",
   " Close Polygon",
   " Done     ",
   " Num. Table",
   " Bar Chart "
};

/*-------------------------------------------------------------*/
/*      Global variables declaration                           */
/*-------------------------------------------------------------*/
XPoint points[100], Points4[4];
Region reg[BOXT+BOXS], regs[10],reghc;
XEvent event;

/*-------------------------------------------------------------*/
/*   x11intf                      X11 interface routine        */
/*-------------------------------------------------------------*/
/*   CHANGE       for Ultrix or Silicon Graphics               */
/*   void x11intf_(xv,yv,z, action, col, zv, string)           */
/*   OTHERWISE                                                 */
/*   void x11intf(xv,yv,z, action, col, zv)                    */
/*-------------------------------------------------------------*/
void x11intf(xv,yv,z, action, col , zv)
int xv[],yv[],zv[],*z,*action;
int *col ;
/*   CHANGE       for Ultrix or Silicon Graphics               */
/*                add the following line                       */
/* char string [];                                             */


{
/*   CHANGE       for Ultrix or Silicon Graphics               */
/*                delete the following line                    */
   extern char string[80];

   int *x, *y,np,iact;
   XGCValues values;

   static int Angle1 = 23040 ;
   static int Angle2 = 23040 ;

   static int xlx, yly, dx, dy ;         /* more statics for     */
   static int start_xlx, start_yly;      /* line drawing         */
   static Boolean loop0,loop1;

   int n,ix,iy;
   int xb,yb,i,j,ii;
   int Iz;
   XSetWindowAttributes attr;
   XSizeHints hints;
   char messaggio[80];
   int asc,des,lar,alt;

   khead = zv[0];
   ipres = zv[1];
   jpres = zv[2];
   kprog = zv[3];
   deltax = zv[4];
   deltay = zv[5];
   kscrx = zv[6];
   kscry = zv[7];

   x = xv;
   y = yv;
   switch (*action)
   {
   case -1:
      sprintf(filcon,"%s",strtok(string," "));
      if(!IsAFile(filcon))
      {
         *z = -1;
      }
      break;

   case 0:                               /* close the window     */
      if(FMesg)
         XFreeFont(display, fontStruct);
      if(FMenu)
         XFreeFont(display, fntmenu);
      if(FHelp)
         XFreeFont(display, fnttab);
      XFreeGC(display,gc);
      XCloseDisplay(display);
      break;

   case 1:                               /* open Xwindow         */
      fprintf(stderr,"opening display \n");
      if ((display = XOpenDisplay(NULL)) != NULL ) 
      {
	  First = True;
        khead = 1;
        wi  = *x       ;
        he = *y       ;
        if ( wi > DisplayWidth(display,0) - BW ) wi = DisplayWidth(display,0) - BW ;
        else if(wi < kscrx)
		wi = kscrx;
        if ( he > DisplayHeight(display,0) - BH ) he = DisplayHeight(display,0) - BH ;
        else if(he < kscry)
		he = kscry;
        height = he - deltay;
        width = wi - deltax;
        drawing.x = deltax;
        drawing.y = deltay;
        drawing.width = width;
        drawing.height = height;
        all.x = 0;
        all.y = 0;
        all.width = wi;
        all.height = he;
	  for (i=BOXES;i<BOXT;i++)
          {
	  ybox[i] = he - 22;
	  xbox[i] = (wi - 639) + xbox[i];
          }
        xbox[BOXES-2] = wi - lbox[BOXES-2];           /* HardCopy */

        attr.backing_store = Always;                 /* Redraw when Obscured */
        attr.background_pixel = BlackPixel(display,0);
        mwindow = XCreateWindow(
                  display,
                  RootWindow(display,DefaultScreen(display)),
                  0,0,
                  wi,he,
                  0,CopyFromParent,CopyFromParent,CopyFromParent,
                  CWBackingStore|CWBackPixel,&attr);
                  
/*      Set title of window */

            if(kprog %10 == 1)
	       XStoreName(display,mwindow,
            "X-SIR92 Status Window & MOLPLO Graphics\0");
        else if(kprog %10 == 2)
	       XStoreName(display,mwindow,
             "X-SIRPOW.92 Status Window & MOLPLO Graphics\0");
        else
	       XStoreName(display,mwindow,
             "X-MOLPLO Graphics\0");

/*      No Resize      */

        hints.flags = PPosition | PSize | PMinSize | PMaxSize ;
        hints.x = 0;
        hints.y = 0;
        hints.width = wi;
        hints.height = he;
        hints.min_width = wi;
        hints.min_height = he;
        hints.max_width = wi;
        hints.max_height = he;
        XSetSizeHints(display,mwindow,&hints,XA_WM_NORMAL_HINTS);
        XMapWindow(display,mwindow);

        /* define colors */

        DefColors();           

        /* create graphic context */

        values.foreground = white;
        values.background = black;
   
        gc = XCreateGC(display,mwindow,GCForeground|GCBackground,&values);

        sprintf(messaggio,"Selecting fonts & colors. Please wait...\0") ;
        Bottom(messaggio);

        /* load fonts  */

        LoadFonts(); 
        if(FMesg)
            XSetFont(display,gc,fontStruct->fid);

         /* Define Defaults for Switches */

         IfEdges = 0;
         IfScale = 1;
         IfCont  = 0;

         /*     Save title in Titolo  */

         strcpy(Titolo,string);
   
         /*     Set up the top-left part of the window */

         TopLeft();


         xv[0]  = wi;
         yv[0]  = he;
   
         XFlush(display);
         sleep(3);
      }

      else
      {
         fprintf(stderr,"\n  x11intf: cannot open display \n\n");
         *z = -1;
      }
      break;

   case 2:                               /* draw to x,y          */
      start_xlx = xv[0];
      start_yly = yv[0];
      xlx       = xv[1];
      yly       = yv[1];
      dx  = start_xlx - xlx;
      dy  = start_yly - yly;
      if((xv[0] <= deltax) || (xv[1] <= deltax) ||
         (yv[0] <= deltay) || (yv[1] <= deltay))
           khead = 1;
      XSetClipRectangles(display,gc,0,0,&drawing,1,Unsorted);
      if (dx == 0 && dy == 0)
         XDrawPoint(display, mwindow, gc, xlx, yly);
      else
         XDrawLine(display, mwindow, gc, xlx, yly,start_xlx,start_yly);
      XSetClipRectangles(display,gc,0,0,&all,1,Unsorted);

      break;

   case 3:                                  /* reset title and menu window */
      TopLeft();
      break;

   case 5:                               /* event loop           */
      
      Iz = *z;
      loop0 = True;
      while (loop0)
      {
      loop0 = False;
      if ( ipres == 0 )                      
      {
        loop1 = True;
        while (loop1)                     /* wait for Button click */
        {
           DrawMenu(Iz);
     
           Sorcio (&ix,&iy);
           i = KeBox(ix,iy);
           if(i >= 0)
              loop1 = False;
        }
      }
      else
        i = ipres + 9;

      *z = i - 9;
      switch (i)
      {
         case 10:                          /* Plot Style */
         case 11:
         case 12:
         case 13:
         case 14:
         case 15:
           curstyl = *z;
         break;

         case 28:                          /* Cell edges on/off */
            strbox[28] = strapp[IfEdges];
            IfEdges = 1 - IfEdges;
		RedrawTasto(28);
         break;

         case 29:                          /* Cell content on/off */
            strbox[29] = strapp[2+IfCont];
            if ( IfCont == 0 )
            {
               sprintf(messaggio,"Running. Please wait...\0") ;
               Bottom(messaggio);
            }
            IfCont = 1 - IfCont;
		RedrawTasto(29);
         break;

         case 30:                               /* Scale  on/off       */
            strbox[30] = strapp[4+IfScale];
            IfScale = 1 - IfScale;
		RedrawTasto(30);
         break;

         case 31:                          /* Delete atom  */
            iact = 0;
            SelectAtms(xv,yv,&iact,1);
            if(iact == -1)
               loop0 = True;
         break;

         case 32:                          /* Select */
            Seleziona(xv,yv,&iact);
            if(iact != -1)
               *z = iact;
            else 
               *z = curstyl;
         break;

         case 33:                          /* View Plane */
            ViewPlane(&iact);
            if(iact != -1)
                  *z = iact ;
            else 
               loop0 = True;
         break;

         case 34:                          /* Relabel */
            RelAt (xv,yv,&iact);
            if(iact == -1)
               loop0 = True;
         break;

         case 35:                   /* Regroup          */
            sprintf(messaggio,"Running. Please wait...\0") ;
            Bottom(messaggio);
         break;

         case 38:                  /* Help window  */
            CreateHelp();
         break;

         case 39:                  /* HardCopy */
            HardCopy();
         break;

         case 40:                  /* Geometry */
            Geometry(xv,yv,&iact);
            if(iact != -1)
                  *z = iact ;
            else 
               loop0 = True;
         break;

         default:
         	if((i == BOXES+30) || (i == BOXES+34))         /* Up */
            	*z = -1;
	   	else if((i == BOXES+31)||(i == BOXES+35))      /* Down */
            	*z = 1;
	   	else if((i == BOXES+32)||(i == BOXES+36))     /* Cancel */
            {
                  strbox[BOXES+33] = strapp[10];
                  khead = 0;
                  histo = False;
                  *z = 0;
                  ClearBottom();
            }
	   	else if(i == BOXES + 33)                /* Bar Chart /Num.Table */
            {
                  *z = 2;
                  if(strbox[i] == strapp[9])
                    strbox[i] = strapp[10];
                  else
                    strbox[i] = strapp[9];
            }
         }
      }

      for (ii=nbox0;ii<=nboxn;ii++)
         XDestroyRegion(reg[ii]);

      break;

   case 6:                     /* clear screen         */
      XClearArea(display, mwindow, deltax,deltay,0,0,0);
      XFlush(display);
      break;

   case 7:                    /* flush buffer */
      XFlush(display);
      break;

   case 8:                    /* Set Switches values to default value */
      strbox[28] = strapp[1];
      strbox[29] = strapp[3];
      strbox[30] = strapp[4];
      IfEdges = 0;
      IfScale = 1;
      IfCont  = 0;
      break;

   case 9:                     /* draw a string*/

      xb = xv[0];
      yb = yv[0];
      if ( xb > 0 )
         PrintImageMess(xb,yb,string);

      else

      {
         xb = - xb;
         PrintImageMess(xb,yb,string);
	   First = True;
         TopLeft();

         XSetForeground(display, gc, colors[*col]);
         j=8;
         xb = 20;
         if(FMesg)
            asc = fontStruct->max_bounds.ascent / 3;
         else
            asc = 4;
         yb = *y - asc;
         points[0].x = xb;
         points[0].y = yb+asc;
         points[1].x = xb+40;
         points[1].y = yb+asc;
         points[2].x = xb+40;
         points[2].y = yb+2*asc;
         points[3].x = xb+60;
         points[3].y = yb;
         points[4].x = xb+40;
         points[4].y = yb-2*asc;
         points[5].x = xb+40;
         points[5].y = yb-asc;
         points[6].x = xb;
         points[6].y = yb-asc;
         points[7].x = xb;
         points[7].y = yb+asc;
         XFillPolygon(display,mwindow, gc, points, j,Nonconvex,CoordModeOrigin);
         XSetForeground(display, gc, colors[old_color]);
      }
      break;

   case 10:                /* draw a circle*/
      if((*x <= deltax) || (*y <= deltay))
           khead = 1;
      XSetClipRectangles(display,gc,0,0,&drawing,1,Unsorted);
      XDrawArc(display, mwindow, gc, *x, *y , *z ,*z ,Angle1,Angle2);
      XSetClipRectangles(display,gc,0,0,&all,1,Unsorted);
      break;

   case 11:                /* fill a circle*/
      /* Angle1,Angle2) = (starting degree, degrees in anti-clockwise) */
      if((*x <= deltax) || (*y <= deltay))
           khead = 1;
      XSetClipRectangles(display,gc,0,0,&drawing,1,Unsorted);
      XFillArc(display, mwindow, gc, *x,*y , *z ,*z ,Angle1,Angle2);
      XSetClipRectangles(display,gc,0,0,&all,1,Unsorted);
      break;

   case 12:                /* set ForeGround color */
      XSetForeground(display, gc, colors[*col]);
      old_color = *col;
      break;

   case 13:                /* draw z-points polygon using color col */
      new_color = *col;
      XSetClipRectangles(display,gc,0,0,&drawing,1,Unsorted);
      XSetForeground(display, gc, colors[new_color]);
      for (np=1;np<*z;np++)
         XDrawLine(display, mwindow, gc, xv[np-1], yv[np-1], xv[np], yv[np]);
      XSetForeground(display, gc, colors[old_color]);
      XSetClipRectangles(display,gc,0,0,&all,1,Unsorted);
      break;

   case 14:               /* fill z-points polygon using color col */
      new_color = *col;
      XSetClipRectangles(display,gc,0,0,&drawing,1,Unsorted);
      XSetForeground(display, gc, colors[new_color]);
      for (np=0;np<*z;np++)
      {
         points[np].x = xv[np];
         points[np].y = yv[np];
      }
      XFillPolygon(display, mwindow, gc, points, *z,Nonconvex,CoordModeOrigin);
      XSetForeground(display, gc, colors[old_color]);
      XSetClipRectangles(display,gc,0,0,&all,1,Unsorted);
      break;

   case 15:               /* Print a message at the bottom of the window */
      XSetForeground(display, gc, black);
      XFillRectangle(display,mwindow,gc,deltax,he-10,width,10);
      Bottom(string);
      break;

   case 16:               /* Store atomic species in array Species */

      NumSpec = *z;
      strcpy(Species,string);
      strcpy(Species_copy,string);
      break;

   default:               /* big trouble          */
      fprintf(stderr,"Error in x11intf action= %d", *action);
      break;

   }
   zv[0] = khead;
   zv[1] = ipres;
   zv[2] = jpres;
   zv[3] = kprog;
}
/*-------------------------------------------------------------*/
/*   fdate                        Time & date routine          */
/*-------------------------------------------------------------*/
/*   CHANGE       for VMS this subroutine has to be deleted    */
/*   CHANGE       for Ultrix or Silicon Graphics               */
/*   void fdate_(datestring)                                   */
/*   OTHERWISE                                                 */
/*   void fdate(datestring)                                    */
/*-------------------------------------------------------------*/
#include <time.h>
void fdate(datestring)
char datestring[];
{
      time_t timer;
      timer = time(&timer);
      strftime(datestring,19,"%d.%m.%y %H:%M:%S", localtime(&timer));
}
/*-------------------------------------------------------------*/
/*   DefColors: definisce i colori utilizzati dal programma    */
/*   nel caso di schermo a colori o monocromatico.             */
/*-------------------------------------------------------------*/
DefColors()
{
   typedef struct {
                    int red;
                    int green;
                    int blue;
                   }  rgbcol ;
   int depth,i,class;
   XColor rgb;
   XVisualInfo template,*info;
   Colormap cmap;
   rgbcol rgbcolor[MAX_COLORS];

   /* set up MOLPLO color table - color definition in 
      /usr/lpp/X11/lib/X11/rgb.txt   */


    depth  = DisplayPlanes(display, 0);          /* Color Planes */

    cmap   = DefaultColormap(display, 0);

    Monocro = False;

    template.screen = DefaultScreen(display);
    template.depth  = depth;
 
    info = XGetVisualInfo(display,VisualScreenMask | VisualDepthMask,
           &template,&i);
    if(i > 0)
         class = info[0].class;
    XFree(info);
    if(depth >= 4)                     /* >= 16 Colors or GrayScale */
    {
       if(class >= 2)                     /* Color Display */
       {
          rgbcolor[0].red =  0;
          rgbcolor[0].green =  0;
          rgbcolor[0].blue =  0;
          rgbcolor[1].red =  21588;
          rgbcolor[1].green =  28784;
          rgbcolor[1].blue =  43690;
          rgbcolor[2].red =  0;
          rgbcolor[2].green =  44975;
          rgbcolor[2].blue =  5140;
          rgbcolor[3].red =  31354;
          rgbcolor[3].green =  50629;
          rgbcolor[3].blue =  52685;
          rgbcolor[4].red =  52685;
          rgbcolor[4].green =  0;
          rgbcolor[4].blue =  0;
          rgbcolor[5].red =  35723;
          rgbcolor[5].green =  0;
          rgbcolor[5].blue =  35723;
          rgbcolor[6].red =  62708;
          rgbcolor[6].green =  42148;
          rgbcolor[6].blue =  24672;
          rgbcolor[7].red =  64250;
          rgbcolor[7].green =  60395;
          rgbcolor[7].blue =  55255;
          rgbcolor[8].red =  28784;
          rgbcolor[8].green =  32896;
          rgbcolor[8].blue =  37008;
          rgbcolor[9].red =  31868;
          rgbcolor[9].green =  39064;
          rgbcolor[9].blue =  54227;
          rgbcolor[10].red =  0;
          rgbcolor[10].green =  65535;
          rgbcolor[10].blue =  0;
          rgbcolor[11].red =  39064;
          rgbcolor[11].green =  62965;
          rgbcolor[11].blue =  65535;
          rgbcolor[12].red =  65535;
          rgbcolor[12].green =  0;
          rgbcolor[12].blue =  0;
          rgbcolor[13].red =  65535;
          rgbcolor[13].green =  0;
          rgbcolor[13].blue =  65535;
          rgbcolor[14].red =  65535;
          rgbcolor[14].green =  65535;
          rgbcolor[14].blue =  0;
          rgbcolor[15].red =  65535;
          rgbcolor[15].green =  65535;
          rgbcolor[15].blue =  65535;
       }
       else if(class < 2)                     /* Gray Scale Display */
       {
           rgbcolor[0].red = 0;
           rgbcolor[0].green = 0;
           rgbcolor[0].blue = 0;
           rgbcolor[8].red = 7168;
           rgbcolor[8].green = 7168;
           rgbcolor[8].blue = 7168;
           rgbcolor[5].red = 54528;
           rgbcolor[5].green = 54528;
           rgbcolor[5].blue = 54528;
           rgbcolor[10].red = 34304;
           rgbcolor[10].green = 34304;
           rgbcolor[10].blue = 34304;
           rgbcolor[12].red = 54016;
           rgbcolor[12].green = 54016;
           rgbcolor[12].blue = 54016;
           rgbcolor[2].red = 33536;
           rgbcolor[2].green = 33536;
           rgbcolor[2].blue = 33536;
           rgbcolor[3].red = 39680;
           rgbcolor[3].green = 39680;
           rgbcolor[3].blue = 39680;
           rgbcolor[14].red = 57600;
           rgbcolor[14].green = 57600;
           rgbcolor[14].blue = 57600;
           rgbcolor[1].red = 17408;
           rgbcolor[1].green = 17408;
           rgbcolor[1].blue = 17408;
           rgbcolor[4].red = 50944;
           rgbcolor[4].green = 50944;
           rgbcolor[4].blue = 50944;
           rgbcolor[11].red = 44544;
           rgbcolor[11].green = 44544;
           rgbcolor[11].blue = 44544;
           rgbcolor[7].red = 57856;
           rgbcolor[7].green = 57856;
           rgbcolor[7].blue = 57856;
           rgbcolor[9].red = 24832;
           rgbcolor[9].green = 24832;
           rgbcolor[9].blue = 24832;
           rgbcolor[6].red = 57088;
           rgbcolor[6].green = 57088;
           rgbcolor[6].blue = 57088;
           rgbcolor[13].red = 55296;
           rgbcolor[13].green = 55296;
           rgbcolor[13].blue = 55296;
           rgbcolor[15].red = 65280;
           rgbcolor[15].green = 65280;
           rgbcolor[15].blue = 65280;
       }
       for (i = 0; i < MAX_COLORS; i++ )
       {
          rgb.red = rgbcolor[i].red;
          rgb.green = rgbcolor[i].green;
          rgb.blue = rgbcolor[i].blue;
          if(XAllocColor(display,cmap,&rgb) == 1)
                colors[i] = rgb.pixel;
          else
                Monocro = True;
        }
    }
    if ((depth < 4) || Monocro)  
    {
        colors[0] = BlackPixel(display,0);
        menucont = 15;
        menulab = 0;
        botrig = 0;
        toplef = 15;
        titcol = 0;
        helpcol = 15;
        for ( i = 1; i < MAX_COLORS; i++ )
           colors[i] = WhitePixel(display, 0);
        fprintf(stderr,"Allocated 2 Colors on Monochrome Display\n");
    }
    else
    {
      if(class >= 2)
        fprintf(stderr,"Allocated %d Colors on Color Display\n",MAX_COLORS);
      else
        fprintf(stderr,"Allocated %d Colors on GrayScale Display\n",MAX_COLORS);
    }
    black = colors[0];
    white = colors [MAX_COLORS-1];
}
/*-------------------------------------------------------------*/
/*   LoadFonts: load fonts for menu keys, messages & info      */
/*-------------------------------------------------------------*/
void LoadFonts()
{
  int lar,count,i,jc[3],l1[100][2],l2[100][2],l3[100][3];
  int lar1,lar2,lar3,w,scelto;
  XFontStruct *info;
  char **fontnames,font1[80],font2[80],font3[80];

  FMenu = FMesg = FHelp = FALSE;
  lar1 = ( kscrx - deltax) / 30;
  lar2 = 16;
  lar3 = ( kscrx - deltax) / 70;
  if(lar1 < 6)
      lar1 = 6;
  if(lar3 < 6)
      lar3 = 6;

  jc[0] = jc[1] = jc[2] = 0;

  /* CHANGE in case of difficulties try the other possibility ("courB*") */
  /* List Fonts "courB*" */

/*
  fontnames = XListFonts(display,"courB*",200,&count);
*/
  fontnames = XListFonts(display,"*courier-bold*",200,&count);

  /* restringe la ricerca a quelle che non superano una larghezza prefissata */

  for (i=0;i<count;i++)
  {
      if((info = XLoadQueryFont(display,fontnames[i])) != NULL)
      {
         XGetFontProperty(info,XA_WEIGHT,&w);
         lar = info->max_bounds.width;
         if(lar <= lar3)                   
         {
            FHelp = True;
            l3[jc[0]][0] = lar;
            l3[jc[0]][1] = i;
            l3[jc[0]++][2] = w;
         }
      }
   }

   /* tra quelle eventualmente trovate sceglie la piu' larga */
   /* a parita' di larghezza, sceglie il peso maggiore (bold) */

   if(FHelp)
   {
      scelto = 0;
      for(i=1;i<jc[0];i++)
      {
         if(abs(l3[i][0] - lar3) < abs(l3[scelto][0] - lar3))
             scelto = i;
         else if((abs(l3[i][0] - lar3) == abs(l3[scelto][0] - lar3)) &&
             (l3[i][2] >= l3[scelto][2]))
             scelto = i;
      }
      sprintf(font3,"%s",fontnames[l3[scelto][1]]);  /* font per help & info */
   }

  /* CHANGE in case of difficulties try the other possibility ("timBI*") */
  /* List Fonts "timBI*" */

/*
   fontnames = XListFonts(display,"timBI*",200,&count);
*/
   fontnames = XListFonts(display,"*times-bold-i*",200,&count);
   if ( count == 0 )
   {
  /* CHANGE in case of difficulties try the other possibility ("courB*") */
/*
        fontnames = XListFonts(display,"courB*",200,&count);
*/
        fontnames = XListFonts(display,"*courier-bold*",200,&count);
        lar1 = (kscrx - deltax)/80;
        lar2 = deltax/14;
        if(lar1 < 6)
           lar1 = 6;
        if(lar2 < 6)
           lar2 = 6;
   }

  /* restringe la ricerca a quelle che non superano una larghezza prefissata */

   for (i=0;i<count;i++)
   if((info = XLoadQueryFont(display,fontnames[i])) != NULL)
   {
     lar = info->max_bounds.width;
     if(lar <= lar1)                     
     {
         FMesg = True;
         l1[jc[1]][0] = lar;
         l1[jc[1]++][1] = i;
     }
     if(lar <= lar2)                    
     {
         FMenu = True;
         l2[jc[2]][0] = lar;
         l2[jc[2]++][1] = i;
         sprintf(font2,"%s",fontnames[i]);
      }
   }

   /* tra quelle eventualmente trovate sceglie la piu' larga */

   if(FMesg)
   {
       scelto = 0;
       for(i=0;i<jc[1];i++)
         if(abs(l1[i][0] - lar1) < abs(l1[scelto][0] - lar1)) 
            scelto = i;
       sprintf(font1,"%s",fontnames[l1[scelto][1]]);   /* Font per messaggi */
   }
   if(FMenu)
   {
       scelto = 0;
       for(i=0;i<jc[2];i++)
         if(abs(l2[i][0] - lar2) < abs(l2[scelto][0] - lar2)) 
            scelto = i;
       sprintf(font2,"%s",fontnames[l2[scelto][1]]);   /* Font per tasti */
   }
   XFreeFontNames(fontnames);

   /* tenta di caricare la font (se l'ha trovata) */

   if(FMesg)
   {
   if((fontStruct = XLoadQueryFont(display,font1)) != NULL)
       fprintf(stderr,"Loaded messages font %s\n",font1);
   else
   {
       fprintf(stderr,"Using default font for messages\n");
       FMesg = False;
   }
   }

   if(FMenu)
   {
   if((fntmenu = XLoadQueryFont(display,font2)) != NULL)
       fprintf(stderr,"Loaded menu keys font %s\n",font2);
    else
   {
       FMenu = False;
       fprintf(stderr,"Using default font for menu keys\n");
   }
   }
   if(FHelp)
   {
    if((fnttab = XLoadQueryFont(display,font3)) != NULL)
       fprintf(stderr,"Loaded info font %s\n",font3);
    else
   {
       FHelp = False;
       fprintf(stderr,"Useing default font for help & info\n");
   }
   }
   if(!FHelp || !FMenu || !FMesg)
       FHelp = FMenu = FMesg = FALSE;
}
/*-------------------------------------------------------------*/
/*   CreateHelp                create help window           */
/*-------------------------------------------------------------*/
void CreateHelp ()
{
   int npag,xb,yb,i;
   char messaggio[200],*str;
   static Boolean loop;
   FILE *fp;

   XClearArea(display, mwindow, deltax+1,deltay+1,0,0,0);

   if ( FHelp )
      XSetFont(display,gc,fnttab->fid);
/*      XSetFont(display,gc,fontStruct->fid); */

   XSetForeground(display, gc, colors[helpcol]);

   /*   Create message to display.  */

   xb = 85;
   yb = 40;
   npag = 0;

   fp = fopen(filcon,"r");
   if(fp != NULL)
   {
      if(fgets(messaggio,200,fp) == NULL)
      {
         sprintf(messaggio,"Read error - file %s\0",filcon);
         Bottom(messaggio);
      }
      else
      {
         while(True)
         {
           if((str = fgets(messaggio,200,fp)) != NULL)
           {
            str[strlen(messaggio)-1] = ' ';
            XDrawString(display,mwindow,gc,xb,yb+14,
                        messaggio,strlen(messaggio));
            yb=yb+25;  
           }
           if(yb >= he - 40 || str == NULL)
           {
               npag++;
               sprintf(messaggio,"Page %d",npag);
               Bottom(messaggio);
               DrawMenu(7);         /* Create "More" and "Cancel" Buttons */
               loop = True;         /* Manage Buttons  */
               while (loop)
               {
                  Sorcio(&xb,&yb);
                  i = KeBox(xb,yb);
                  if(i >= BOXES+10)
                     loop = False;
               }
               if(i == BOXES+11)
                  goto fine;
               else
               {
                  XClearArea(display, mwindow, deltax+1,deltay+1,0,0,0);
                  if ( FHelp )
                     XSetFont(display,gc,fnttab->fid);
/*                     XSetFont(display,gc,fontStruct->fid);*/
                  XSetForeground(display, gc, colors[helpcol]);
                  xb = 85;
                  yb = 40;
                  if(str == NULL)           /* EOF */
                  {
                     npag = 0;
                     rewind(fp);
                     fgets(messaggio,200,fp);
                  }
               }
           }
         }
                  
      }
fine:
      loop = False;
      fclose(fp);
   }
   else
   {
      sprintf(messaggio,"Open error - File is %s\0",filcon) ;
      Bottom(messaggio);
      DrawMenu(6);                  /* Create "Cancel" Button */
      loop = True;         /* Manage Button  */
      while (loop)
      {
          Sorcio(&xb,&yb);
          i = KeBox(xb,yb);
          if(i != -1)
             loop = False;
      }
   }

}
/*-------------------------------------------------------------*/
/*   Tasto: disegna un tasto tridimensionale                   */
/*-------------------------------------------------------------*/

void Tasto (xb,yb,widb,heib)
int xb,yb,widb,heib;
{
   int j;

   j=6;
   XSetForeground(display, gc, colors[botrig]);
   points[0].x = xb;
   points[0].y = yb+heib;
   points[1].x = xb+3;
   points[1].y = yb+heib-3;
   points[2].x = xb+widb-3;
   points[2].y = yb+heib-3;
   points[3].x = xb+widb-3;
   points[3].y = yb+3;
   points[4].x = xb+widb;
   points[4].y = yb;
   points[5].x = xb+widb;
   points[5].y = yb+heib;
   XFillPolygon(display,mwindow, gc, points, j,Nonconvex,CoordModeOrigin);
   XSetForeground(display, gc, colors[toplef]);
   points[2].x = xb+3;
   points[2].y = yb+3;
   points[5].x = xb;
   points[5].y = yb;
   XFillPolygon(display,mwindow, gc, points, j,Nonconvex,CoordModeOrigin);
}
/*-------------------------------------------------------------*/
/*   C_PrintMess: draws "string" at "xb,yb", color "colo"      */
/*              used to draw keys                              */
/*-------------------------------------------------------------*/
void C_PrintMess(xb,yb,string,colo)
int xb,yb,colo;
char string[];
{
   if ( FMenu )
      XSetFont(display,gc,fntmenu->fid);

   if (colo != -1)
      XSetForeground(display, gc, colors[colo]);
   XDrawString(display, mwindow, gc, xb, yb, string, strlen(string) );
   if (colo != -1)
      XSetForeground(display, gc, colors[old_color]);
   if ( FMesg )
      XSetFont(display,gc,fontStruct->fid);
   XFlush(display);
}
/*-------------------------------------------------------------*/
/*   PrintImageMess: draws "string" at "xb,yb"                 */
/*-------------------------------------------------------------*/
void PrintImageMess(xb,yb,string)
int xb,yb;
char string[];
{
   int i;
   for(i=strlen(string);i<79;i++)
      string[i] = ' ';
   string[79] = '\0';
   if(FHelp && histo)
      XSetFont(display,gc,fnttab->fid);
   XDrawImageString(display, mwindow, gc, xb+deltax, yb, string, 79 );
   if(FMesg)
      XSetFont(display,gc,fontStruct->fid);
   XFlush(display);
}
void TopLeft()
/*-------------------------------------------------------------*/
/*   TopLeft: Reset menu window & print title on the top of    */
/*            the window                                       */
/*-------------------------------------------------------------*/
{
   XSetForeground(display, gc, white);
   XFillRectangle(display,mwindow,gc,deltax+1,0,width-deltax,deltay);
   if(First)
   {
   XFillRectangle(display,mwindow,gc,0,0,wi,deltay);
   XSetForeground(display, gc, black);
   XFillRectangle(display,mwindow,gc,0,deltay+1,deltax,height);
   }
   XSetForeground(display, gc, colors[old_color]);
   C_PrintMess(80+deltax,15,Titolo,titcol);

}
void ClearMenu()
/*-------------------------------------------------------------*/
/*   ClearMenu: Reset menu area & title area for HardCopy      */
/*-------------------------------------------------------------*/
{
   XSetForeground(display, gc, black);
   XFillRectangle(display,mwindow,gc,0,0,wi,deltay+1);
   XFillRectangle(display,mwindow,gc,0,deltay+1,deltax,height);
   XSetForeground(display, gc, colors[old_color]);
   XFlush(display);
}
/*-------------------------------------------------------------*/
/*   DrawMenu: draw menu                                       */
/*-------------------------------------------------------------*/
void DrawMenu(Iz)
int Iz;
{
   int xb,yb,widb,heib,i;

   if ( FMenu )
      XSetFont(display,gc,fntmenu->fid);

   hc = False;
   switch (Iz)
   {
   case 0:                              /* draw menu */
      TopLeft();
      nbox0  = 0 ;
      nboxn  = BOXES - 1 ;
      break;

   case 1:                              /* draw "Quit" */
   case -1:
      TopLeft();
      nbox0  = 9;
      nboxn  = 9 ;
      hc = True;
      break;

   case 2:                              /* draw keys for "Select" */
      nbox0  = BOXES  ;
      nboxn  = BOXES + 4 ;
      break;

   case 3:                              /* draw "Close Polygon" and "Cancel" */
      nbox0  = BOXES + 5;
      nboxn  = BOXES + 6;
      break;

   case 4:                              /* draw keys for "View Plane" */
      nbox0  = BOXES + 7;
      nboxn  = BOXES + 11;
      break;

   case 5:                              /* draw other keys for "View Plane" */
      nbox0  = BOXES + 12;
      nboxn  = BOXES + 16;
      break;

   case 6:                              /* draw key "Cancel" */
      nbox0  = BOXES + 11;
      nboxn  = BOXES + 11;
      break;

   case 7:                              /* draw keys for "Help" */
      nbox0  = BOXES + 10;
      nboxn  = BOXES + 11;
      break;

   case 8:                              /* only for P.C. HardCopy */
      nbox0  = BOXES + 17;
      nboxn  = BOXES + 19;
      break;

   case 9:                              /* draw keys for "Geometry" */
      nbox0  = BOXES + 20;
      nboxn  = BOXES + 24;
      break;

   case 10:                            /* draw other keys for "Geometry" */
      nbox0  = BOXES + 25;
      nboxn  = BOXES + 29;
      break;

   case 11:                           /* Up Down Cancel for Bar Chart  */
      nbox0  = BOXES + 34;
      nboxn  = BOXES + 36;
      break;

   case 12:                           /* Up Down Cancel & Bar Chart  /NumT */
      nbox0  = BOXES + 30;
      nboxn  = BOXES + 33;
      break;

   case 13:                           /* Cancel & Bar Chart  / NumT. */
      nbox0  = BOXES + 32;
      nboxn  = BOXES + 33;
      break;

   default:
      break;
   }


   for (i=nbox0;i<=nboxn;i++)
   {

/*            draws keys           */

      widb=lbox[i];
      heib=20;
	if(First || (Iz != 0))
		RedrawTasto(i);

/*         create regions          */

      Points4[0].x = Points4[3].x = xbox[i];
      Points4[1].x = Points4[2].x = xbox[i]+widb;
      Points4[0].y = Points4[1].y = ybox[i];
      Points4[2].y = Points4[3].y = ybox[i]+heib;
      reg [i] = XPolygonRegion(Points4,4,EvenOddRule);

   }

/*       draws "HCopy" key         */

   if(hc)
   {
      i = BOXES - 2;
      widb=lbox[i];
      heib=20;
	RedrawTasto(i);
      Points4[0].x = Points4[3].x = xbox[i];
      Points4[1].x = Points4[2].x = xbox[i]+widb;
      Points4[0].y = Points4[1].y = ybox[i];
      Points4[2].y = Points4[3].y = ybox[i]+heib;
      reghc = XPolygonRegion(Points4,4,EvenOddRule);
    }

   if(First && (Iz == 0))
	First = False;
   if ( FMesg )
      XSetFont(display,gc,fontStruct->fid);

}
/*-------------------------------------------------------------*/
/*   Bottom: Print message "string" at the bottom of the window*/
/*-------------------------------------------------------------*/
void Bottom(string)
char string[];
{
   ClearBottom();
   XSetForeground(display, gc, colors[helpcol]);
   if ( FMesg )
      XSetFont(display,gc,fontStruct->fid);
   XDrawImageString(display,mwindow,gc,85,he-4,string,strlen(string));
   XFlush(display);
}
/*-------------------------------------------------------------*/
/*   Sorcio: Wait for mouse "click"                            */
/*-------------------------------------------------------------*/
void Sorcio(ix,iy)
int *ix,*iy;
{

   XSelectInput(display,mwindow,ButtonPressMask);
   XWindowEvent(display,mwindow,ButtonPressMask,&event);
   *ix = event.xbutton.x;
   *iy = event.xbutton.y;
}
/*-------------------------------------------------------------*/
/*   KeBox: returns key pressed                                */
/*-------------------------------------------------------------*/
int KeBox(ix,iy)
int ix,iy;
{
   int i;

   if(hc && XPointInRegion(reghc,ix,iy))
      return(39);
   for (i=nbox0;i<=nboxn;i++)
   {
      if(XPointInRegion(reg[i],ix,iy))
         return(i);
   }
   return (-1);
}
/*-------------------------------------------------------------*/
/*   KeSpecies: returns selected species                       */
/*-------------------------------------------------------------*/
int KeSpecie(ix,iy)
int ix,iy;
{
   int i;

   for (i=0;i<NumSpec;i++)
   {
      if(XPointInRegion(regs[i],ix,iy))
         return(i);
   }
   return (-1);
}
/*-------------------------------------------------------------*/
/*   DrawSpecies: draws keys for scelta della specie           */
/*-------------------------------------------------------------*/
void DrawSpecies()
{
   int xb,yb,i,widb,heib,colo;
   char *str2;

   xb = 350;
   yb = he - 22;
   widb=30;
   heib=20;
   strcpy(Species,Species_copy);
   str2 = strtok(Species,"+");
   for (i=0;i<NumSpec;i++)
   {
      xb = xb + widb + 10;
      if(Monocro)
         colo = 0;
      else
         colo = i+1;
      XSetForeground(display, gc, colors[colo]);
      XFillRectangle(display,mwindow,gc,xb+1,yb+1,widb-2,heib-2);
      Tasto (xb,yb,widb,heib);
      Points4[0].x = Points4[3].x = xb;
      Points4[1].x = Points4[2].x = xb+widb;
      Points4[0].y = Points4[1].y = yb;
      Points4[2].y = Points4[3].y = yb+heib;
      regs [i] = XPolygonRegion(Points4,4,EvenOddRule);
      XSetForeground(display, gc, colors[menucont]);
      XDrawRectangle(display,mwindow,gc,xb,yb,widb,heib);

      C_PrintMess(xb+7,yb+14,str2,menulab);
      str2 = strtok(NULL,"+");
   }
}
/*-------------------------------------------------------------*/
/*   SelectAtms :  selects atoms                               */
/*-------------------------------------------------------------*/
void SelectAtms(xv,yv,iact,NumMsg)
int xv[],yv[],*iact,NumMsg;
{
   int ix,iy,i,nel,Iz;
   Boolean loop = True;
   char messaggio [80];

   if ( *iact == 0 )
   {
         ClearBottom();
         switch(NumMsg)
         {
              case 1:
              sprintf(messaggio,"Click on atoms to delete\0") ;
              break;

              case 2:
              sprintf(messaggio,"Click on wanted atom(s)\0") ;
              break;

              case 3:
              sprintf(messaggio,"Click on three atoms\0") ;
              break;

              case 4:
              sprintf(messaggio,"Click on four atoms\0") ;
              break;

              case 7:
              case 8:
              sprintf(messaggio,"Click on couples of atoms\0") ;
              break;
         }
         Bottom(messaggio);

      }
      *iact = 0;
      strbox[BOXES+5] = strapp[8];
      strbox[BOXES+6] = strapp[6];
      Iz = 3;
      if ( NumMsg == 3 ||  NumMsg == 4) 
         Iz = 6;
      DrawMenu(Iz);

   nel = 0;
   while(loop)
   {
      loop = False;
      Sorcio (&ix,&iy);
      i = KeBox(ix,iy);
      if (i < 0)
      {
         xv[nel] = ix;
         yv[nel] = iy;
         nel++;
         loop = True;
         if ( ( NumMsg >= 2 &&  NumMsg <= 4) && nel == NumMsg )
            { 
              xv[nel] = -1;
              loop = False;
            } 
      }

      xv[nel] = -1;
      if (i == nboxn)
          *iact = -1;
   }
      ClearBottom();
}
/*-------------------------------------------------------------*/
/*   ClearBottom: clear area at bottom of the window           */
/*-------------------------------------------------------------*/
void ClearBottom()
{
   int i;
   char string[80];

   /* Clear messages area */

   for(i=0;i<79;i++)
      string[i] = ' ';
   string[79] = '\0';
   if ( FMesg )
      XSetFont(display,gc,fontStruct->fid);
   XDrawImageString(display,mwindow,gc,85,he-4,string,79);

   /* Clear keys area */

   XClearArea(display, mwindow, deltax+1,he-25,0,0,0);
}
/*-------------------------------------------------------------*/
/*   Seleziona: seleziona un gruppo di atomi                   */
/*-------------------------------------------------------------*/
void Seleziona (xv,yv,iact)
int xv[],yv[],*iact;

{
   Boolean loop;
   int ix,iy,i,ii,nel,ix1,iy1;
   char messaggio [80];

   ClearBottom();
   DrawMenu(2);
   loop = True;
   while(loop)
   {
      Sorcio(&ix,&iy);
      i = KeBox(ix,iy);
      if(i >= 0)
         loop = False;
   }
   if(i < BOXES+3)
   {
      if(i == BOXES)                    /* To delete */
         *iact = -23;
      else if(i == BOXES+1)             /* To hide */
         *iact = 23;
      else if(i == BOXES+2)             /* To display */
         *iact = 223;
      sprintf(messaggio,"Draw a polygon around atoms ...");
      Bottom(messaggio);
      strbox [BOXES+5] = strapp[7];
      strbox [BOXES+6] = strapp[6];
      DrawMenu(3);
      nel = 0;
      ix1 = -1;
      iy1 = -1;
      loop = True;
      XSetForeground(display, gc, colors[7]);
      while(loop)
      {
         Sorcio(&ix,&iy);
         ii = KeBox(ix,iy);
         if(ii >= 0)
            loop = False;
         else
         {
            xv[nel] = ix;
            yv[nel] = iy;
            nel++;
            if(nel > 1)
               XDrawLine(display,mwindow,gc,ix,iy,ix1,iy1);
            ix1 = ix;
            iy1 = iy;
         }
      }

      if (ii == BOXES+6)                               /* cancel */
         *iact = -1;
      else                                             /* close polygon */
      {
         xv [nel] = xv [0];
         yv [nel] = yv [0];
         xv [nel+1] = -1;
         yv [nel+1] = -1;
      }
   }
   else if(i == BOXES+3)                              /* Show all   */
      *iact = 123;
   else if (i == BOXES+4)                             /* Cancel */
      *iact = -1;

   if(*iact == -1)
     ClearBottom();
}
Boolean IsAFile(path)
char *path;
{
	FILE *fpt;

	if((fpt = fopen(path,"r")) == NULL)
		return(False);
      fclose(fpt);
	return(True);
}
/*-------------------------------------------------------------*/
/*   HardCopy: hardcopy della parte grafica su stampante       */
/*-------------------------------------------------------------*/
void HardCopy()
{
   char messaggio[80], id[100];
   FILE *fp;

   fp = fopen(filcon,"r");
   if(fp != NULL)
   {
      if(fgets(messaggio,80,fp) != NULL)
      {
         sprintf(id,messaggio,mwindow);
         ClearMenu();
         system(id);
         First = True;
         khead = 0;
      }
      else
      {
         sprintf(messaggio,"Read error - file %s\0",filcon);
         Bottom(messaggio);
      }
      fclose(fp);
   }
   else
   {
      sprintf(messaggio,"Open error - file %s\0",filcon) ;
      Bottom(messaggio);
   }
}
/*-------------------------------------------------------------*/
/*   ViewPlane: select view plane                              */
/*-------------------------------------------------------------*/
void ViewPlane(vp)
int *vp;
{
   Boolean loop1,loop2;
   int ii,ix,iy;

   *vp = -1;
   ClearBottom();
   loop2 = True;
   while (loop2)
   {
      DrawMenu(4);
      loop1 = True;
      while(loop1)
      {
         Sorcio(&ix,&iy);
         ii = KeBox(ix,iy);
         if(ii >= 0)
            loop1 = False;
      }
      if(ii <= 50)
      {
         *vp = ii - 16;
         loop2 = False;
      }
      else if(ii == 51)                             /* More ... */
      {
         DrawMenu(5);
         loop1 = True;
         while(loop1)
         {
            Sorcio(&ix,&iy);
            ii = KeBox(ix,iy);
            if(ii >= 0)
               loop1 = False;
         }
         if(ii <= 55)
         {
            *vp = ii -16;
            loop2 = False;
         }
         else if(ii == 57)                             /* Cancel */
            loop2 = False;
      }
      else if(ii == 52)                                /* Cancel */
         loop2 = False;
      ClearBottom();
   }
   if(*vp < 0)
     ClearBottom();
}
/*-------------------------------------------------------------*/
/*   Geometry: supply geometry informations                   */
/*-------------------------------------------------------------*/
void Geometry(xv,yv,vp)
int xv[],yv[],*vp;
{
   Boolean loop1,loop2;
   int ii,ix,iy,iz,iact,nmsg;

   *vp = -1;
   if(jpres == 0)
      ClearBottom();
   loop2 = True;
   iz = 9;
   while (loop2)
   {
      if ( jpres == 0 )
      {
         DrawMenu(iz);
         loop1 = True;
         while(loop1)
         {
            Sorcio(&ix,&iy);
            ii = KeBox(ix,iy);
            if(ii >= 0)
               loop1 = False;
         }
      }
      else
      {
         ii = jpres + 9;
         strbox[BOXES + 33] = strapp[10];
      }
         
      if((ii >= BOXES + 20) && (ii <= BOXES + 22))
      {                                  /* Distances,Angles,Torsion Angles */
         nmsg = ii - BOXES - 18;
         if(jpres == 0)
             iact = 0;
         else
             iact = 1;
         SelectAtms(xv,yv,&iact,nmsg);
         if(iact == -1)  
         {
             ipres = 0;
             jpres = 0;
         }
         else
             *vp = (nmsg - 1) * 100 + 31;
         loop2 = False;
      }
      else if(ii == BOXES + 23)          /* More ... */
         iz = 10;
      else if(ii == BOXES + 28)          /* More ... */
         iz = 9 ;
      else if((ii >= BOXES + 25) && (ii <= BOXES + 26))
      {                               /* Add Bonds,Delete Bonds */
         nmsg = ii - BOXES - 18;
         if(jpres == 0)
             iact = 0;
         else
             iact = 1;
         SelectAtms(xv,yv,&iact,nmsg);
         if(iact == -1)  
         {
             ipres = 0;
             jpres = 0;
         }
         else
            *vp = (nmsg - 1) * 100 + 31;
         loop2 = False;
      }
      else if(ii == BOXES + 27)       /* Info   */
      {
         *vp = (ii - BOXES - 19) * 100 + 31;
         histo = True;
         loop2 = False;
      }
      else if(ii == BOXES + 29 || ii == BOXES+24)       /* Cancel */
          loop2 = False;
      
   }
   if(*vp < 0)
     ClearBottom();
}
/*-------------------------------------------------------------*/
/*   RelAt: Relabel atoms                                      */
/*-------------------------------------------------------------*/
void RelAt(xv,yv,iact)
int *iact,xv[],yv[];
{
   char messaggio[80];
   int ii,xb,yb;
   Boolean loop1;

   sprintf(messaggio,"Select atom to Relabel\0") ;
   Bottom(messaggio);
   DrawMenu(6);
   Sorcio(&xb,&yb);
   ii = KeBox(xb,yb);
   if(ii < 0)
   {

      xv[0] = xb;
      yv[0] = yb;

      sprintf(messaggio,"Select new atomic specie\0");
      Bottom(messaggio);

      DrawSpecies();

      loop1 = True;
      while (loop1)
      {
         Sorcio(&xb,&yb);
         ii = KeSpecie(xb,yb);
         if(ii >= 0)
         {
            yv[1] = ii+1;
            loop1 = False;
         }
      }
      *iact = ii + 1;
      for(ii=0;ii<NumSpec;ii++)
         XDestroyRegion(regs[ii]);
   }  
   else
   {
      *iact = -1;
      ClearBottom();
   }
}
/*-------------------------------------------------------------*/
/*   RedrawTasto                                               */
/*-------------------------------------------------------------*/
void RedrawTasto(i)
int i;
{
	int widb,heib;

      widb=lbox[i];
      heib=20;
      XSetForeground(display, gc, white);
      XFillRectangle(display,mwindow,gc,xbox[i]+1,ybox[i]+1,widb-2,heib-2);
      Tasto (xbox[i],ybox[i],widb,heib);
      XSetForeground(display, gc, colors[menucont]);
      XDrawRectangle(display,mwindow,gc,xbox[i],ybox[i],widb,heib);
      C_PrintMess(xbox[i]+6,ybox[i]+14,strbox[i],menulab);
}
